#include "game.h"
#include "level.h"
#include "frame.h"

//===== Player implementation =======//

num player::world_angle() const
{
    num arc = space_platform_arc();
    num angle = this->platform * arc;

    if(this->airborne()) {
        /* Time spent rotating; caps at the full rotation time */
        num t = num_clamp(this->jump_t, 0, TIME_ROTATION);
        angle += this->jump_dir * arc * t / TIME_ROTATION;
    }

    return angle;
}

vec2 player::world_angle_vector() const
{
    num angle = this->world_angle();
    return vec2(num_cos(angle), num_sin(angle));
}


vec3 player::pos() const
{
    /* Local position without accounting for current platform's angle */
    vec3 pos(0, -space_platform_distance() + this->height, this->z);

    return vec_rotate_around_z(pos, this->world_angle_vector());
}

vec3 player::forward() const
{
    return vec3(0, 0, 1);
}

vec3 player::backward() const
{
    return -this->forward();
}

vec3 player::up() const
{
    return vec_rotate_around_z(vec3(0, 1, 0), this->world_angle_vector());
}

vec3 player::down() const
{
    return -this->up();
}

image_t *player::anim()
{
        static image_t *sprite_sheet_run[] = {
                &img_erik_running_01, &img_erik_running_02, &img_erik_running_03,
                &img_erik_running_04, &img_erik_running_05, &img_erik_running_06,
                &img_erik_running_07, &img_erik_running_08, &img_erik_running_09,
                &img_erik_running_10, &img_erik_running_11, &img_erik_running_12,
                &img_erik_running_13, &img_erik_running_14, &img_erik_running_15,
                &img_erik_running_16, &img_erik_running_17, &img_erik_running_18,
                &img_erik_running_19, &img_erik_running_20, &img_erik_running_21,
                &img_erik_running_22, &img_erik_running_23, &img_erik_running_24,
                &img_erik_running_25, &img_erik_running_26
        };
        static image_t *sprite_sheet_jump[] = {
                &img_erik_jumping_01, &img_erik_jumping_02, &img_erik_jumping_03,
                &img_erik_jumping_04, &img_erik_jumping_05, &img_erik_jumping_06,
                &img_erik_jumping_07, &img_erik_jumping_08, &img_erik_jumping_09,
                &img_erik_jumping_10, &img_erik_jumping_11, &img_erik_jumping_12,
                &img_erik_jumping_13, &img_erik_jumping_14, &img_erik_jumping_15,
                &img_erik_jumping_16, &img_erik_jumping_17, &img_erik_jumping_18,
                &img_erik_jumping_19, &img_erik_jumping_20, &img_erik_jumping_21,
                &img_erik_jumping_22, &img_erik_jumping_23, &img_erik_jumping_24,
                &img_erik_jumping_25, &img_erik_jumping_26, &img_erik_jumping_27,
                &img_erik_jumping_28, &img_erik_jumping_29, &img_erik_jumping_30,
                &img_erik_jumping_31, &img_erik_jumping_32, &img_erik_jumping_33,
                &img_erik_jumping_34, &img_erik_jumping_35, &img_erik_jumping_36,
                &img_erik_jumping_37, &img_erik_jumping_38, &img_erik_jumping_39,
                &img_erik_jumping_40, &img_erik_jumping_41, &img_erik_jumping_42,
        };

        this->frame += 1;
        if (this->stance == Running) {
                if ((this->frame >> 1) >= 26)
                        this->frame = 0;
                return sprite_sheet_run[this->frame >> 1];
        }
        if ((this->frame >> 1) >= 43)
                this->frame = 0;
        return sprite_sheet_jump[this->frame >> 1];
}


//======= Game implementation =======//

//======= General utilities =======//


static vec2 platform_rotations[PLATFORM_COUNT];
static num platform_distance;

num space_platform_distance(void)
{
    return platform_distance;
}

/* We can't use a constructor function because g++ already generates a call to
   the default constructor, which would run after us and override the data. */
void game_init(void)
{
    num arc = space_platform_arc();

    for(int i = 0; i < PLATFORM_COUNT; i++) {
        num angle = arc * i - (arc / 2);
        platform_rotations[i] = vec2(num_cos(angle), num_sin(angle));
    }

    platform_distance = LEVEL_RADIUS * num_cos(arc / 2);
}

struct prect space_platform_position(int platform_id, num z)
{
    struct prect r;
    vec3 radius(0, -LEVEL_RADIUS, z);

    int angle_l = platform_id;
    int angle_r = (platform_id + 1) % PLATFORM_COUNT;

    r.nl = r.fl = vec_rotate_around_z(radius, platform_rotations[angle_l]);
    r.nr = r.fr = vec_rotate_around_z(radius, platform_rotations[angle_r]);

    r.fl.z += SECTION_LENGTH;
    r.fr.z += SECTION_LENGTH;
    return r;
}
